<?php

namespace Drupal\media_fotoweb;

use Fotoweb\FotowebClient as FotowebAPIClient;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use kamermans\OAuth2\OAuth2Handler;

/**
 * Class FotowebClient.
 */
class FotowebClient {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * The Guzzle Service Client.
   *
   * @var \GuzzleHttp\Command\ServiceClientInterface
   */
  public $client;

  /**
   * The current configuration.
   *
   * @var array
   */
  protected $configuration;

  /**
   * The rendition service URL.
   *
   * @var string
   */
  protected $renditionService;

  /**
   * FotowebClient constructor.
   *
   * @param array $configuration
   *   The guzzle service configuration.
   */
  public function __construct(array $configuration) {
    $this->createClientFromConfiguration($configuration);
  }

  /**
   * Creates a FotowebClient from configuration.
   *
   * @param array $configuration
   *   A Fotoweb client configuration array.
   */
  public function createClientFromConfiguration(array $configuration) {
    if ($configuration['authType'] === 'oauth2') {
      $configuration['client_config']['auth'] = 'oauth';
      $configuration['grantType'] = $configuration['grantType'] ?? 'client_credentials';

      if ($configuration['grantType'] === 'authorization_code' || (isset($configuration['persistenceType']) && $configuration['persistenceType'] === 'user_token_persistence')) {        $tokenPersistence = \Drupal::service('media_fotoweb.user_token_persistence');
        // For a user based authentication we assume it is for the selection
        // widget. Therefore use the selection widget credentials.
        $configuration['clientId'] = $configuration['selectionWidgetClientId'];
        $configuration['clientSecret'] = $configuration['selectionWidgetClientSecret'];
      }
      else {
        $tokenPersistence = \Drupal::service('media_fotoweb.api_token_persistence');
      }
      $configuration['persistenceProvider'] = $tokenPersistence;
    }
    $this->setConfiguration($configuration);
    $this->client = new FotowebAPIClient($configuration);
  }

  /**
   * Sets the rendition service.
   *
   * @param string $rendition_service
   *   The rendition service URL.
   */
  public function setRenditionService($rendition_service) {
    $this->renditionService = $rendition_service;
  }

  /**
   * Returns the rendition service.
   *
   * @return string
   *   The rendition service URL.
   */
  public function getRenditionService() {
    return $this->renditionService;
  }

  /**
   * Sets the configuration.
   *
   * @param array $configuratiton
   *   The configuration.
   */
  public function setConfiguration($configuration) {
    $this->configuration = $configuration;
  }

  /**
   * Returns the configuration.
   *
   * @return array
   *   The configuration.
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * Return the oAuth access token.
   *
   * @return string|null
   *   The access token.
   */
  public function getAccessToken() {
    $token = NULL;

    if ($this->client->getAuthenticationMiddleware()) {
      $middleware = $this->client->getAuthenticationMiddleware();
      if ($middleware instanceof OAuth2Handler) {
        $token = $middleware->getAccessToken();
      }
    }

    return $token;
  }

  /**
   * Fetches the API Descriptor.
   *
   * @return mixed
   *   The API descriptor.
   */
  public function fetchApiDescriptor() {
    return $this->client->getApiDescriptor();
  }

  /**
   * Fetches the renditions ervice.
   *
   * @return mixed|null
   *   The rendition service or NULL on failur.
   */
  public function fetchRenditionService() {
    $renditionService = NULL;

    $apiDescriptor = $this->fetchApiDescriptor();
    if (!empty($apiDescriptor)) {
      $services = $apiDescriptor->offsetGet('services');

      if (!empty($services['rendition_request'])) {
        $renditionService = $services['rendition_request'];
      }
    }

    return $renditionService;
  }

  /**
   * Checks whether the API is authenticated.
   *
   * @return bool
   *   True if the API is authenticated.
   */
  public function isApiAuthenticated() {
    if (\Drupal::state()->get('media_fotoweb.oauth2_token')) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Forward methods to the Fotoweb Client.
   *
   * @param string $name
   *   Name of the method.
   * @param array $arguments
   *   Method arguments.
   *
   * @return mixed
   *   The method return value.
   */
  public function __call($name, array $arguments) {
    return call_user_func_array([$this->client, $name], $arguments);
  }

}
